#include "ell/NeighborGenerator.hh"
#include "ell/State.hh"
#include <biu/assertbiu.hh>

namespace ell {

	NG_Random::NeighborList::NeighborList(const State* _origin) 
		:origin(_origin)
	{}
	
	NG_Random::NeighborList::~NeighborList() {}
	
	State*
	NG_Random::NeighborList::first(State::NeighborList::ItState** itstate) const
	{
		State* newstate = NULL;
		do newstate = origin->getRandomNeighbor(newstate);
		while (newstate == NULL);
		return newstate;
	}
	
	State*
	NG_Random::NeighborList::next(	State::NeighborList::ItState* itstate, 
									State* elem) const
	{
		assertbiu(elem != NULL, "Elem is not allowed to be NULL");

		delete elem;
		State* nextstate = NULL;
		do nextstate = origin->getRandomNeighbor(nextstate);
		while (nextstate == NULL);
		return nextstate;
		
		// TODO: reuse elem for better performance, but segfaults
//		State* nextstate = NULL;
//		do nextstate = origin->getRandomNeighbor(elem);
//		while (nextstate == NULL);
//		elem = nextstate;
//		return elem;
	}
	
	State::NeighborListPtr 
	NG_Random::getNeighborList(const State* const s) const
	{
		return State::NeighborListPtr(new NG_Random::NeighborList(s));
	}
	
	State::NeighborListPtr 
	NG_SuccessiveRandom::getNeighborList(const State* const s) const
	{
		return s->getRandomNeighborList();
	}

	State::NeighborListPtr 
	NG_Successive::getNeighborList(const State* const s) const
	{
		return s->getNeighborList();
	}
	
} // namespace ell
