#ifndef SC_LISTING_HH_
#define SC_LISTING_HH_

#include "ell/SC_Counting.hh"
#include <list>

namespace ell 
{
	
	/*!
	 * This subclass of of WalkCollector keeps a list of all added States.
	 * 
	 * @author Daniel Maticzka
	 */
	class SC_Listing : public SC_Counting
	{
	public:
	
		typedef std::list<State*> List;
		 
	protected:
	
		  //! the internal storage of all added states 
		List stateList;
		
	public:
	
		SC_Listing();
		~SC_Listing();

		  //! Adds State to the state list.
		  //! @param s the added State
		virtual void add(const State& s);
		
		  //! Returns number of added States.
		  //! @return the number of added states
		virtual size_t size() const;
		 
		  //! Returns last added State.
		  //! @return the last added State
		virtual const State* const getLastAdded() const;
		
		///////////  LIST HANDLING  ////////////////////////
	
		  //! Returns a list of all accepted States.
		  //! @return the State list
		virtual const List& getList() const;
		
	};
} // namespace ell

#endif /*SC_LISTING_HH_*/
