#ifndef SC_LISTINGCOMPR_HH_
#define SC_LISTINGCOMPR_HH_

#include "ell/SC_Counting.hh"
#include <list>

namespace ell 
{
	
	/*!
	 * This subclass of WalkCollector keeps a list of all added States and
	 * stores the compressed string representation of each State.
	 * 
	 * @author Daniel Maticzka
	 * @author Martin Mann
	 */
	class SC_ListingCompr : public SC_Counting
	{
	public:
	
		typedef std::list<CSequence> List;
		 
	protected:
	
		  //! the internal storage of all added states 
		List stateList;

	public:
	
		SC_ListingCompr();
		~SC_ListingCompr();

		  //! This function stores the compressed string representation  
		  //! in a list.
		  //! @param s the added State
		virtual void add(const State& s);
		
		///////////  LIST HANDLING  ////////////////////////
	
		  //! Returns the list of compressed string representations of all 
		  //! managed States.
		  //! @return the compressed sequence list
		virtual const List& getList() const;

	};
} // namespace ell

#endif /*SC_LISTINGCOMPR_HH_*/
