#include "ell/State.hh"
#include "ell/protein/S_LP_PullM.hh"
#include <biu/LatticeProtein_Ipnt.hh>
#include <biu/PullMoveSet.hh>
#include <biu/LatticeDescriptorCUB.hh>
#include <biu/assertbiu.hh>

namespace ell
{

S_LP_PullM::S_LP_PullM(		biu::LatticeProtein_I* _latProt,
		 					biu::LatticeMoveSet* _moveSet)
	: S_LP(_latProt, _moveSet)
	{}

S_LP_PullM::S_LP_PullM( const S_LP_PullM& s)
	: S_LP( s.latProt->clone(), s.moveSet->clone() )
	{}

S_LP_PullM::~S_LP_PullM()
	{}

State* 
S_LP_PullM::clone() const
{
	assertbiu(latProt != NULL , "no lattice protein (NULL)");
	assertbiu(moveSet != NULL , "no move set (NULL)");
	return new S_LP_PullM(*this);
}

	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
		/* Access to a compressed sequence representation of the state.
		 * @return the compressed sequence representation
		 */
	CSequence  
	S_LP_PullM::compress(void) const {
		return latProt->getLatticeModel()->getDescriptor()->
					getAlphabet()->compress(latProt->getMoveSeqAbs());
	}
	
		/* Access to a compressed sequence representation of the state.
		 * @param toFill a data structure to write the compressed 
		 *               representation too
		 * @return the compressed sequence representation
		 */
	CSequence&  
	S_LP_PullM::compress(CSequence& toFill) const {
		toFill = latProt->getLatticeModel()->getDescriptor()->
					getAlphabet()->compress(latProt->getMoveSeqAbs());
		return toFill;
	}
	
		/* Uncompresses a compressed sequence representation into a new
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @param toFill a state object to uncompress too or NULL if a new 
		 *               object has to be created
		 * @return new State object that is encoded in cseq or NULL in error
		 *         case.
		 */
	State*  
	S_LP_PullM::uncompress(const CSequence& cseq, State* toFill) const {
        
		const std::string newSeq =
			latProt->getLatticeModel()->getString(
					latProt->getLatticeModel()->getDescriptor()->
					getAlphabet()->decompress(cseq, latProt->getLength()-1));
		
		if (toFill==NULL) {
			// create new object
			biu::LatticeProtein_I* newProt = latProt->clone();
			newProt->setMoveStrAbs(newSeq);
			toFill = new S_LP_PullM(newProt, moveSet->clone());
		}
		else
		{
			S_LP_PullM* s = dynamic_cast<S_LP_PullM*>(toFill);
			assertbiu(s != NULL, "casting to S_LP_PullM failed");
			s->latProt->setMoveStrAbs(newSeq);
			toFill = s;
		}
		
        return toFill;
	}
	
		/* Uncompresses a compressed sequence representation into a this
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @return this or NULL in error case
		 */
	State*  
	S_LP_PullM::uncompress(const CSequence& cseq) {
		return uncompress(cseq, this);
	}

} // namespace ell
