#ifndef S_LP_PULLM_HH_
#define S_LP_PULLM_HH_

#include "ell/protein/S_LP.hh"
#include <biu/LatticeModel.hh>
#include <biu/LatticeProtein_I.hh>
#include <biu/LatticeMoveSet.hh>
#include <string>

namespace ell
{

class S_LP_PullM : public ell::S_LP
{
	
public:
	
	  //! Constructs a State using a LatticeProtein and a PullMoveSet.
	  //! @param latProt 	the lattice protein used by this State
	  //! @param moveSet	the move set used by this State	
	S_LP_PullM(	biu::LatticeProtein_I* latProt,
				biu::LatticeMoveSet* moveSet);
	
	S_LP_PullM( const S_LP_PullM&);
	
	virtual ~S_LP_PullM();

	virtual State* clone() const;
	
	//////////////  COMPRESSED REPRESENTATION  ////////////////////////////
	
		/*! Access to a compressed sequence representation of the state.
		 * @return the compressed sequence representation
		 */
	virtual CSequence compress(void) const;
	
		/*! Access to a compressed sequence representation of the state.
		 * @param toFill a data structure to write the compressed 
		 *               representation too
		 * @return the compressed sequence representation
		 */
	virtual CSequence& compress(CSequence& toFill) const;
	
		/*! Uncompresses a compressed sequencce representation into a new
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @param toFill a state object to uncompress too or NULL if a new 
		 *               object has to be created
		 * @return new State object that is encoded in cseq or NULL in error
		 *         case.
		 */
	virtual State* uncompress(const CSequence& cseq, State* toFill ) const;
	
		/*! Uncompresses a compressed sequencce representation into a this
		 * State object.
		 * @param cseq the compressed sequence representation of a state
		 * @return this or NULL in error case
		 */
	virtual State* uncompress(const CSequence& cseq);
	
	virtual unsigned int getMinimalDistance(const State& state2) const { 
		return UINT_MAX; 
	};
};

} // end namespace ell
#endif /*S_LP_PULLM_HH_*/
